/*
 * BackchannelUART.c
 *
 *  Created on: Jun 21, 2019
 *      Author: hdevore
 */

#include <msp430.h>
#include "BackchannelUART.h"

unsigned char readValue;

void InitializeBackchannelUART(void)
{
    // The UART data transmission line is normally held at a high voltage level when it’s not transmitting data

    TXD_HI;
    SET_TXD_AS_OUTPUT;


    SET_RXD_PULLUP;
    SET_RXD_AS_INPUT;
    EN_RXD_PUD;

    //P1SEL &= 0b11111001;
    P1SEL2 |= 0b00000110;
    P1SEL |= 0b00000110;
    //P1SEL &= 0b11111100;

    // For a given BRCLK clock source, the baud rate used determines the required division factor N:
    // N = fBRCLK / Baud rate
    // The division factor N is often a non-integer value thus at least one divider and one modulator stage is used to meet the factor as closely as possible.
    // If N is equal or greater than 16 the oversampling baud rate generation mode can be chosen by setting UCOS16.
    // In the low-frequency mode, the integer portion of the divisor is realized by the prescaler:
    // UCBRx = INT(N)
    // and the fractional portion is realized by the modulator with the following nominal formula:
    // UCBRSx = round( ( N – INT(N) ) × 8 )
    // Incrementing or decrementing the UCBRSx setting by one count may give a lower maximum bit error for any given bit. To determine if this is the case, a detailed error calculation must be performed for each bit for each UCBRSx setting.

    // USCI_A0 control register 0 UCA0CTL0 Read/write 060h Reset with PUC
    // USCI_A0 control register 1 UCA0CTL1 Read/write 061h 001h with PUC
    // USCI_A0 Baud rate control register 0 UCA0BR0 Read/write 062h Reset with PUC
    // USCI_A0 baud rate control register 1 UCA0BR1 Read/write 063h Reset with PUC
    // USCI_A0 modulation control register UCA0MCTL Read/write 064h Reset with PUC
    // USCI_A0 status register UCA0STAT Read/write 065h Reset with PUC
    // USCI_A0 receive buffer register UCA0RXBUF Read 066h Reset with PUC
    // USCI_A0 transmit buffer register UCA0TXBUF Read/write 067h Reset with PUC
    // USCI_A0 Auto baud control register UCA0ABCTL Read/write 05Dh Reset with PUC
    // USCI_A0 IrDA transmit control register UCA0IRTCTL Read/write 05Eh Reset with PUC
    // USCI_A0 IrDA receive control register UCA0IRRCTL Read/write 05Fh Reset with PUC
    // SFR interrupt enable register 2 IE2 Read/write 001h Reset with PUC
    // SFR interrupt flag register 2 IFG2 Read/write 003h 00Ah with PUC

    // The UART data transmission line is normally held at a high voltage level when it’s not transmitting data
    //  To start the transfer of data, the transmitting UART pulls the transmission line from high to low for one clock cycle. When the receiving UART detects the high to low voltage transition, it begins reading the bits in the data frame at the frequency of the baud rate.
    // Parity describes the evenness or oddness of a number. The parity bit is a way for the receiving UART to tell if any data has changed during transmission. Bits can be changed by electromagnetic radiation, mismatched baud rates, or long distance data transfers. After the receiving UART reads the data frame, it counts the number of bits with a value of 1 and checks if the total is an even or odd number. If the parity bit is a 0 (even parity), the 1 bits in the data frame should total to an even number. If the parity bit is a 1 (odd parity), the 1 bits in the data frame should total to an odd number. When the parity bit matches the data, the UART knows that the transmission was free of errors. But if the parity bit is a 0, and the total is odd; or the parity bit is a 1, and the total is even, the UART knows that bits in the data frame have changed.
    // To signal the end of the data packet, the sending UART drives the data transmission line from a low voltage to a high voltage for at least two bit durations.

    /*
    // Enable all clocks and CPU
    SCG1 = 0;
    SCG0 = 0;
    OSCOFF = 0;
    CPUOFF = 0; */

    // I will go ahead and include parity so that we have a regular number of bits to send (a whole byte)

 /*   UCPEN = 1;  // Enable parity
    UCPAR = 1;  // Even parity
    UCMSB = 1;  // MSB first
    UC7BIT = 0; // 8 bit data
    UCSPB = 1;  // 2 stop bits
    UCMODE1 = 0;
    UCMODE0 = 0; // regular UART mode
    UCSYNC = 0; // Asynchronous mode */

    UCA0CTL0 = 0b11101000; // control register 0 of USCI_A0, set to the conditions listed above

   /* UCSSEL1 = 1;
    UCSSEL0 = 1;  // USCI clock from SMCLK
    UCRXEIE = 0;  // reject erroneous characters
    UCBRKIE = 0;  // received break characters do not set UCA0RXIFG
    UCDORM = 0;   // not in dormant mode
    UCTXADDR = 0; // data transmitted is data, not an address
    UCTXBRK = 0;  // next data is not a break
    UCSWRST = 0;  // USCI reset is disabled, so the USCI is useable
    */

    UCA0CTL1 = 0b11000000; // control register 1 of USCI_A0 is set to the conditions listed above

    IE2 |= 0b00000001; // enable USCI_A0 recieve interupt

    UCA0MCTL &= 0b11111110; // disable oversampling in the modulation control register

    //  Set the baud rate:
    // 1,000,000 9600 104 1 0 -0.5 0.6 -0.9 1.2
    // to get a Baud rate of 9600 with maximum TX error of -0.5 to 0.6, and a maximum RX error of -0.9 to 1.2,
    // BRCLK frequency needs to be 1,048,576 Hz
    UCA0BR0 = 104;
    UCA0BR1 = 104;  // UCBRx = 104
    UCA0MCTL = 0b00000010; // UCBRFx = 0, UCBRSx = 1, UCOS16 = 0;

    // SMCLK configured through BCSCTL2
    BCSCTL2 &= 11111001; // the divider for SMCLK is 1, so it remains at 1MHz

    UCA0STAT = 0b00000000; // clears the status register

    //P1SEL |= BIT1 | BIT2;

}

//unsigned char ReadValue(void)
#pragma vector = USCIAB0RX_VECTOR
// RXIFG interrupt service routine
__interrupt void RecieveInterrupt(void)
{

    // The USCI module is enabled by clearing the UCSWRST bit and the receiver is ready and in an idle state. The receive baud rate generator is in a ready state but is not clocked nor producing any clocks.
    // The falling edge of the start bit enables the baud rate generator and the UART state machine checks for a valid start bit. If no valid start bit is detected the UART state machine returns to its idle state and the baud rate generator is turned off again. If a valid start bit is detected a character will be received.
    // When the idle-line multiprocessor mode is selected with UCMODEx = 01 the UART state machine checks for an idle line after receiving a character. If a start bit is detected another character is received. Otherwise the UCIDLE flag is set after 10 ones are received and the UART state machine returns to its idle state and the baud rate generator is turned off.
    // Glitch suppression prevents the USCI from being accidentally started. Any glitch on UCAxRXD shorter than the deglitch time tτ (approximately 150 ns) will be ignored by the USCI and further action will be initiated
    // When a glitch is longer than tτ or a valid start bit occurs on UCAxRXD, the USCI receive operation is started and a majority vote is taken as shown in Figure 15-9. If the majority vote fails to detect a start bit the USCI halts character reception

    // The UCAxRXIFG interrupt flag is set each time a character is received and loaded into UCAxRXBUF. An interrupt request is generated if UCAxRXIE and GIE are also set. UCAxRXIFG and UCAxRXIE are reset by a system reset PUC signal or when UCSWRST = 1. UCAxRXIFG is automatically reset when UCAxRXBUF is read
    // When UCAxRXEIE = 0 erroneous characters will not set UCAxRXIFG.
    // When UCDORM = 1, non-address characters will not set UCAxRXIFG in multiprocessor modes. In plain UART mode, no characters will set UCAxRXIFG
    // When UCBRKIE = 1 a break condition will set the UCBRK bit and the UCAxRXIFG flag.


    //while (UCA0STAT != 0b00000000); // check that there are no error flags or busy flags, etc.


    readValue = UCA0RXBUF;

    // Clear the interrupt flag
    IFG2 &= 0b11111110;
}

void SendValue(unsigned char value)
{
    // The USCI module is enabled by clearing the UCSWRST bit and the transmitter is ready and in an idle state. The transmit baud rate generator is ready but is not clocked nor producing any clocks.

    // A transmission is initiated by writing data to UCAxTXBUF. When this occurs, the baud rate generator is enabled and the data in UCAxTXBUF is moved to the transmit shift register on the next BITCLK after the transmit shift register is empty. UCAxTXIFG is set when new data can be written into UCAxTXBUF.

    // Transmission continues as long as new data is available in UCAxTXBUF at the end of the previous byte transmission. If new data is not in UCAxTXBUF when the previous byte has transmitted, the transmitter returns to its idle state and the baud rate generator is turned off.

    UCA0TXBUF = value;
    while (UCA0STAT & UCBUSY);
}
